#
# NOTE: Modified for truncating.
#
# Examples of Use: name_block(profile, business, '[0,40]', 'box1_1', 'box1_2')
#
# This is an example of a global truncation, which means each box supplied to name_block will be
# trunated at 40 chars. So in this example, box1_1 and box1_2 will both be truncated at 40 chars.
#
# Individual box truncation rather than a global truncation can also be performed.
#
# Example: name_block(profile, business, 'box1_1[0,10]', 'box1_2[0,20]')
#
# In this example, box1_1 is truncated at 10 chars while box1_2 is truncated at 20 chars.
#
# You can also use BOTH global and individual box truncation. Individual box truncation overrides the
# global truncation in such a case.
#
# Example: name_block(profile, business, '[0,40]', 'box1_1[0,10]', 'box1_2[0,20]', 'box1_3')
#
# In this example, box1_1 is truncated at 10 chars, box1_2 is truncated at 20 chars, and box1_3 is truncated at 
# the global setting of 40 chars.
#
# Generic dataflow for 1, 2, 3 or 4 line address block.
#
def name_block(profile, business, *lines)
	business = false unless business.kind_of? TrueClass
	name = address1 = address2 = address3 = ''
	global_truncate = false
	truncate0 = truncate1 = 0
	data_flow = Array.new

	if (profile.to_s[2,10] == 'Individual')
		# Decide if profile's business or personal information should be used.
		if (business and !profile.business_name.strip.empty?)
			# Use business information.
			name = profile.business_name
			if (profile.business_address.strip.empty?)
				address1 = profile.address
				address2 = 'Apt. #' + profile.apartment_number unless profile.apartment_number.strip.empty?
				address3 = (profile.city.strip.empty? ? '' : profile.city + ', ') + profile.state + ' ' + profile.zip
			else
				address1 = profile.business_address
				address2 = profile.business_address2
				address3 = (profile.business_city.strip.empty? ? '' : profile.business_city + ', ') + profile.business_state + ' ' + profile.business_zip
			end
		else
			# Using personal information.
			name = profile.first_name + ' ' + profile.middle_initial + ' ' + profile.last_name + ' ' + profile.suffix
			address1 = profile.address
			address2 = 'Apt. #' + profile.apartment_number unless profile.apartment_number.strip.empty?
			address3 = (profile.city.strip.empty? ? '' : profile.city + ', ') + profile.state + ' ' + profile.zip
		end
	elsif profile.to_s[2,6] == 'Entity'
		name = profile.name
		address1 = profile.address
		address2 = profile.address2
		address3 = (profile.city.strip.empty? ? '' : profile.city + ', ') + profile.state + ' ' + profile.zip
	elsif profile.to_s[2,8] == 'Preparer'
		# Use Firm name if available and business = true.
		# Use Preparer name if the Preparer is self-employed and business = false.
		if (!profile.firm_name.strip.empty? and ((profile.self_employed.strip.empty?) or (!profile.self_employed.strip.empty? and business)))
			name = profile.firm_name
		else
			name = profile.first_name + ' ' + profile.middle_initial + ' ' + profile.last_name
		end
		address1 = profile.address
		address3 = (profile.city.strip.empty? ? '' : profile.city + ', ') + profile.state + ' ' + profile.zip	
	elsif profile.to_s[2,3] == 'Fcc'
		# Use entity name if available.
		if (profile.applicant_entity_name.strip.empty?)
			name = profile.applicant_first_name + ' ' + profile.applicant_middle_initial + ' ' + profile.applicant_last_name + ' ' + profile.applicant_suffix
		else
			name = profile.applicant_entity_name
		end
		address1 = profile.applicant_address
		address2 = 'P.O. Box ' + profile.applicant_po_box unless profile.applicant_po_box.strip.empty?
		address3 = (profile.applicant_city.strip.empty? ? '' : profile.applicant_city + ', ') + (profile.applicant_state.strip.empty? ? profile.applicant_country : profile.applicant_state) + ' ' + profile.applicant_zip
	else
		return
	end
	
	data_flow[0] = name.squeeze(' ').strip
	data_flow[1] = address1.squeeze(' ').strip
	data_flow[2] = address2.squeeze(' ').strip
	data_flow[3] = address3.squeeze(' ').strip

	# Check for a global truncate.
	if (lines[0] =~ /\A(\[)(\d+)(\,)(\d+)(\])\z/)
		global_truncate = true
		truncate0 = $2.to_i
		truncate1 = $4.to_i
		lines.delete_at(0)
	end

	# Decide what lines are output.
	case lines.length
		when 1
			data_flow[0] = (name + ' ' + address1 + ' ' + address2 + ' ' + address3).squeeze(' ').strip
		when 2
			data_flow[1] = (address1 + ' ' + address2 + ' ' + address3).squeeze(' ').strip
		when 3
			data_flow[1] = (address1 + ' ' + address2).squeeze(' ').strip
			data_flow[2] = data_flow[3]
		when 4
			if (address2.empty?) 
				data_flow[2] = data_flow[3]
				data_flow[3] = ''
			end
	end

	# Do the truncation if applicable - individual truncates take precedence over a global truncate, then print the lines.
	lines.each_index { |index|
		if (!data_flow[index].empty?)
			if (lines[index] =~ /\A(box\d+_\d+)(\[)(\d+)(\,)(\d+)(\])\z/)
				data_flow[index] = data_flow[index][$3.to_i, $5.to_i]
				lines[index] = $1
			elsif (global_truncate)
				data_flow[index] = data_flow[index][truncate0, truncate1]
			end
			print lines[index], data_flow[index]
		end
	}
end
